#!/usr/bin/env python
from dataclasses import dataclass
from pandas import DataFrame
import xarray as xr
import pandas as pd
import numpy as np
from typing import Tuple, Dict
from numpy import cos, sin, arctan, pi, nan
from functools import partial, wraps
import geopandas as gpd
from geopandas import GeoDataFrame
from shapely.geometry import Polygon, Point
import logging
from nes import *

@dataclass
class Grid:
    centre_lat: float
    centre_lon: float
    dlon: float
    dlat: float
    west: float = None
    south: float = None
        
        
    def calculate_grid_coords(self):
        """
        Use Nes to create a grid representation
        """
        nessy = create_nes(comm=None, info=False, projection=self.projection,
                   centre_lat=self.centre_lat, centre_lon=self.centre_lon,
                   west_boundary=self.west, south_boundary=self.south,
                   inc_rlat=self.dlon, inc_rlon=self.dlat)
        nessy.create_shapefile()
        self.gdf = nessy.shapefile
        self.gdf["grid_cell"] = ["grid_cell_{}".format(i+1) for i in range(len(self.gdf))]

@dataclass
class RotatedGrid(Grid):
    projection: str = "rotated"
        
    def __post_init__(self):
        self.calculate_grid_coords()
    
    def aggregate(self, lon, lat, obs, obserr=None):
        """
        Aggregate
        
        Parameters:
        lon, lat, obs values OPtional obserr
        
        Returns:
        lon, lat, obs values
        """
        if obserr is not None:
            df = pd.DataFrame({"obs":obs, "lon":lon, "lat":lat, "obserr": obserr})
        else:
            df = pd.DataFrame({"obs":obs, "lon":lon, "lat":lat})

        df['coords'] = list(zip(df['lon'],df['lat']))
        df['coords'] = df['coords'].apply(Point)

        points = gpd.GeoDataFrame(df, geometry='coords', crs='epsg:4326')
        
        gdf = self.gdf.sjoin(points)
        gdf = gdf.groupby("grid_cell").mean().reset_index()
        
        if obserr is not None:
            return gdf["lon"].values, gdf["lat"].values, gdf["obs"].values, gdf["obserr"].values
        else:
            return gdf["lon"].values, gdf["lat"].values, gdf["obs"].values
        
        
